package it.polito.po.test;

import java.util.LinkedList;
import java.util.Observable;
import java.util.Observer;

import hydraulic.*;
import junit.framework.TestCase;


public class TestR4_Simulazione extends TestCase {

	private class TestObserver implements Observer{
		public LinkedList<HEvent> events = new LinkedList<HEvent>();
		public void update(Observable arg0, Object e) {
			HEvent event = (HEvent)e;
			events.add(event);
		}
		public HEvent findByElement(Element el){
			for(HEvent ev : events){
				if( ev.element == el ){
					return ev;
				}
			}
			return null;
		}
	}

	public void testElementiSemplici(){
		TestObserver obs = new TestObserver();
		HSystem s = new HSystem(obs);
		Source src = new Source("Src");		
		Tap tap = new Tap("Tap");		
		Sink sink = new Sink("Sink");		
		s.addElement(src);
		s.addElement(tap);
		s.addElement(sink);
		
		src.connect(tap);
		tap.connect(sink);
		
		double flow = 100.0;
		src.setFlow(flow);
		tap.setOpen(true);
		
		s.simulate();
		
		HEvent evt = obs.findByElement(tap);
		
		double inTap = evt.input;
		double outTap = evt.outputs[0];

		evt = obs.findByElement(sink);
		
		double inSink = evt.input;

		assertEquals(flow,inTap,0.01);
		assertEquals(flow,outTap,0.01);
		assertEquals(flow,inSink,0.01);
	}
	
	public void testSplit(){
		TestObserver obs = new TestObserver();
		HSystem s = new HSystem(obs);
		Source src = new Source("Src");		
		Split t = new Split("T");	
		Element sink1 = new Sink("Sink 1");		
		Element sink2 = new Sink("Sink 2");		
		s.addElement(src);
		s.addElement(t);
		s.addElement(sink1);
		s.addElement(sink2);
		
		src.connect(t);
		t.connect(sink1,0);
		t.connect(sink2,1);
		
		double flow = 100.0;
		src.setFlow(flow);
		
		s.simulate();

		HEvent evt = obs.findByElement(t);
		
		double out1 = evt.outputs[0];
		double out2 = evt.outputs[1];

		assertEquals(50,out1,0.01);
		assertEquals(50,out2,0.01);
	}

}
